require 'nats/client'
require 'axon-nats/msg'
require 'axon-nats/error'

include Axon
module Axon
	def self.subscribe(route, &block)
		NATS.subscribe(route) { |msg_s|
			begin
				block.call(AxonMsg.parse(msg_s))
			rescue => e
				STDERR.puts e
				STDERR.puts e.backtrace
			end
		}
	end
	def self.respond(route, &block)
		NATS.subscribe(route) { |msg_s, reply| 
			begin
				NATS.publish(reply, AxonMsg.new(block.call(AxonMsg.parse(msg_s))))
			rescue AxonError => e
				NATS.publish(reply, AxonMsg.new(e.data, e.message))
			rescue => e
				if e.is_a? StandardError
					STDERR.puts e
					STDERR.puts e.backtrace
				end
				NATS.publish(reply, AxonMsg.new({}, e.message))
			end
		}
	end
	def self.request(route, req = {}, &block)
		NATS.request(route, AxonMsg.new(req)) { |msg_s|
			block.call(AxonMsg.parse(msg_s))
		}
	end
	def self.publish(route, msg = {})
		NATS.publish(route, AxonMsg.new(msg))
	end
	def self.method_missing(method, *args)
		NATS.send(method, *args) if NATS.respond_to? method
	end
	def self.method_missing(method, *args, &block)
		NATS.send(method, *args, &block) if NATS.respond_to? method
	end
end
